@@ -3,6 +3,7 @@ from optuna.samplers._base import BaseSampler
 from optuna.samplers._brute_force import BruteForceSampler
 from optuna.samplers._cmaes import CmaEsSampler
 from optuna.samplers._grid import GridSampler
+from optuna.samplers._nsgaiii import NSGAIIISampler
 from optuna.samplers._partial_fixed import PartialFixedSampler
 from optuna.samplers._qmc import QMCSampler
 from optuna.samplers._random import RandomSampler
@@ -21,6 +22,7 @@ __all__ = [
     "IntersectionSearchSpace",
     "MOTPESampler",
     "NSGAIISampler",
+    "NSGAIIISampler",
     "PartialFixedSampler",
     "QMCSampler",
     "RandomSampler",
