@@ -6,6 +6,7 @@ from typing import Optional
 from typing import Tuple
 
 from sqlalchemy import asc
+from sqlalchemy import case
 from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
 from sqlalchemy import DateTime
@@ -55,7 +56,6 @@ class StudyModel(BaseModel):
     def find_or_raise_by_id(
         cls, study_id: int, session: orm.Session, for_update: bool = False
     ) -> "StudyModel":
-
         query = session.query(cls).filter(cls.study_id == study_id)
 
         if for_update:
@@ -69,14 +69,12 @@ class StudyModel(BaseModel):
 
     @classmethod
     def find_by_name(cls, study_name: str, session: orm.Session) -> Optional["StudyModel"]:
-
         study = session.query(cls).filter(cls.study_name == study_name).one_or_none()
 
         return study
 
     @classmethod
     def find_or_raise_by_name(cls, study_name: str, session: orm.Session) -> "StudyModel":
-
         study = cls.find_by_name(study_name, session)
         if study is None:
             raise KeyError(NOT_FOUND_MSG)
@@ -98,7 +96,6 @@ class StudyDirectionModel(BaseModel):
 
     @classmethod
     def where_study_id(cls, study_id: int, session: orm.Session) -> List["StudyDirectionModel"]:
-
         return session.query(cls).filter(cls.study_id == study_id).all()
 
 
@@ -118,7 +115,6 @@ class StudyUserAttributeModel(BaseModel):
     def find_by_study_and_key(
         cls, study: StudyModel, key: str, session: orm.Session
     ) -> Optional["StudyUserAttributeModel"]:
-
         attribute = (
             session.query(cls)
             .filter(cls.study_id == study.study_id)
@@ -132,7 +128,6 @@ class StudyUserAttributeModel(BaseModel):
     def where_study_id(
         cls, study_id: int, session: orm.Session
     ) -> List["StudyUserAttributeModel"]:
-
         return session.query(cls).filter(cls.study_id == study_id).all()
 
 
@@ -152,7 +147,6 @@ class StudySystemAttributeModel(BaseModel):
     def find_by_study_and_key(
         cls, study: StudyModel, key: str, session: orm.Session
     ) -> Optional["StudySystemAttributeModel"]:
-
         attribute = (
             session.query(cls)
             .filter(cls.study_id == study.study_id)
@@ -166,7 +160,6 @@ class StudySystemAttributeModel(BaseModel):
     def where_study_id(
         cls, study_id: int, session: orm.Session
     ) -> List["StudySystemAttributeModel"]:
-
         return session.query(cls).filter(cls.study_id == study_id).all()
 
 
@@ -177,7 +170,7 @@ class TrialModel(BaseModel):
     # to be unique. This is to reduce code complexity as table-level locking would be required
     # otherwise. See https://github.com/optuna/optuna/pull/939#discussion_r387447632.
     number = Column(Integer)
-    study_id = Column(Integer, ForeignKey("studies.study_id"))
+    study_id = Column(Integer, ForeignKey("studies.study_id"), index=True)
     state = Column(Enum(TrialState), nullable=False)
     datetime_start = Column(DateTime)
     datetime_complete = Column(DateTime)
@@ -190,14 +183,21 @@ class TrialModel(BaseModel):
     def find_max_value_trial(
         cls, study_id: int, objective: int, session: orm.Session
     ) -> "TrialModel":
-
         trial = (
             session.query(cls)
             .filter(cls.study_id == study_id)
             .filter(cls.state == TrialState.COMPLETE)
             .join(TrialValueModel)
             .filter(TrialValueModel.objective == objective)
-            .order_by(desc(TrialValueModel.value))
+            .order_by(
+                desc(
+                    case(
+                        {"INF_NEG": -1, "FINITE": 0, "INF_POS": 1},
+                        value=TrialValueModel.value_type,
+                    )
+                ),
+                desc(TrialValueModel.value),
+            )
             .limit(1)
             .one_or_none()
         )
@@ -209,14 +209,21 @@ class TrialModel(BaseModel):
     def find_min_value_trial(
         cls, study_id: int, objective: int, session: orm.Session
     ) -> "TrialModel":
-
         trial = (
             session.query(cls)
             .filter(cls.study_id == study_id)
             .filter(cls.state == TrialState.COMPLETE)
             .join(TrialValueModel)
             .filter(TrialValueModel.objective == objective)
-            .order_by(asc(TrialValueModel.value))
+            .order_by(
+                asc(
+                    case(
+                        {"INF_NEG": -1, "FINITE": 0, "INF_POS": 1},
+                        value=TrialValueModel.value_type,
+                    )
+                ),
+                asc(TrialValueModel.value),
+            )
             .limit(1)
             .one_or_none()
         )
@@ -228,7 +235,6 @@ class TrialModel(BaseModel):
     def find_or_raise_by_id(
         cls, trial_id: int, session: orm.Session, for_update: bool = False
     ) -> "TrialModel":
-
         query = session.query(cls).filter(cls.trial_id == trial_id)
 
         # "FOR UPDATE" clause is used for row-level locking.
@@ -249,7 +255,6 @@ class TrialModel(BaseModel):
         study: Optional[StudyModel] = None,
         state: Optional[TrialState] = None,
     ) -> int:
-
         trial_count = session.query(func.count(cls.trial_id))
         if study is not None:
             trial_count = trial_count.filter(cls.study_id == study.study_id)
@@ -259,7 +264,6 @@ class TrialModel(BaseModel):
         return trial_count.scalar()
 
     def count_past_trials(self, session: orm.Session) -> int:
-
         trial_count = session.query(func.count(TrialModel.trial_id)).filter(
             TrialModel.study_id == self.study_id, TrialModel.trial_id < self.trial_id
         )
@@ -282,7 +286,6 @@ class TrialUserAttributeModel(BaseModel):
     def find_by_trial_and_key(
         cls, trial: TrialModel, key: str, session: orm.Session
     ) -> Optional["TrialUserAttributeModel"]:
-
         attribute = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -296,7 +299,6 @@ class TrialUserAttributeModel(BaseModel):
     def where_trial_id(
         cls, trial_id: int, session: orm.Session
     ) -> List["TrialUserAttributeModel"]:
-
         return session.query(cls).filter(cls.trial_id == trial_id).all()
 
 
@@ -316,7 +318,6 @@ class TrialSystemAttributeModel(BaseModel):
     def find_by_trial_and_key(
         cls, trial: TrialModel, key: str, session: orm.Session
     ) -> Optional["TrialSystemAttributeModel"]:
-
         attribute = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -330,7 +331,6 @@ class TrialSystemAttributeModel(BaseModel):
     def where_trial_id(
         cls, trial_id: int, session: orm.Session
     ) -> List["TrialSystemAttributeModel"]:
-
         return session.query(cls).filter(cls.trial_id == trial_id).all()
 
 
@@ -348,14 +348,12 @@ class TrialParamModel(BaseModel):
     )
 
     def check_and_add(self, session: orm.Session) -> None:
-
         self._check_compatibility_with_previous_trial_param_distributions(session)
         session.add(self)
 
     def _check_compatibility_with_previous_trial_param_distributions(
         self, session: orm.Session
     ) -> None:
-
         trial = TrialModel.find_or_raise_by_id(self.trial_id, session)
 
         previous_record = (
@@ -375,7 +373,6 @@ class TrialParamModel(BaseModel):
     def find_by_trial_and_param_name(
         cls, trial: TrialModel, param_name: str, session: orm.Session
     ) -> Optional["TrialParamModel"]:
-
         param_distribution = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -389,7 +386,6 @@ class TrialParamModel(BaseModel):
     def find_or_raise_by_trial_and_param_name(
         cls, trial: TrialModel, param_name: str, session: orm.Session
     ) -> "TrialParamModel":
-
         param_distribution = cls.find_by_trial_and_param_name(trial, param_name, session)
 
         if param_distribution is None:
@@ -399,7 +395,6 @@ class TrialParamModel(BaseModel):
 
     @classmethod
     def where_trial_id(cls, trial_id: int, session: orm.Session) -> List["TrialParamModel"]:
-
         trial_params = session.query(cls).filter(cls.trial_id == trial_id).all()
 
         return trial_params
@@ -452,7 +447,6 @@ class TrialValueModel(BaseModel):
     def find_by_trial_and_objective(
         cls, trial: TrialModel, objective: int, session: orm.Session
     ) -> Optional["TrialValueModel"]:
-
         trial_value = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -464,7 +458,6 @@ class TrialValueModel(BaseModel):
 
     @classmethod
     def where_trial_id(cls, trial_id: int, session: orm.Session) -> List["TrialValueModel"]:
-
         trial_values = (
             session.query(cls).filter(cls.trial_id == trial_id).order_by(asc(cls.objective)).all()
         )
@@ -527,7 +520,6 @@ class TrialIntermediateValueModel(BaseModel):
     def find_by_trial_and_step(
         cls, trial: TrialModel, step: int, session: orm.Session
     ) -> Optional["TrialIntermediateValueModel"]:
-
         trial_intermediate_value = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -541,7 +533,6 @@ class TrialIntermediateValueModel(BaseModel):
     def where_trial_id(
         cls, trial_id: int, session: orm.Session
     ) -> List["TrialIntermediateValueModel"]:
-
         trial_intermediate_values = session.query(cls).filter(cls.trial_id == trial_id).all()
 
         return trial_intermediate_values
@@ -575,7 +566,6 @@ class VersionInfoModel(BaseModel):
 
     @classmethod
     def find(cls, session: orm.Session) -> "VersionInfoModel":
-
         version_info = session.query(cls).one_or_none()
 
         return version_info
