@@ -28,7 +28,6 @@ STORAGE_MODES = [
 
 @pytest.fixture(scope="session", autouse=True)
 def init_process_group() -> None:
-
     if "OMPI_COMM_WORLD_SIZE" not in os.environ:
         pytest.skip("This test is expected to be launch with mpirun.")
 
@@ -38,12 +37,12 @@ def init_process_group() -> None:
     os.environ["MASTER_ADDR"] = "127.0.0.1"
     os.environ["MASTER_PORT"] = "20000"
 
-    dist.init_process_group("gloo", timeout=datetime.timedelta(seconds=15))  # type: ignore
+    dist.init_process_group("gloo", timeout=datetime.timedelta(seconds=15))
 
 
 def test_torch_distributed_trial_experimental_warning() -> None:
     with pytest.warns(optuna.exceptions.ExperimentalWarning):
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study()
             TorchDistributedTrial(study.ask())
         else:
@@ -53,7 +52,7 @@ def test_torch_distributed_trial_experimental_warning() -> None:
 @pytest.mark.filterwarnings("ignore::optuna.exceptions.ExperimentalWarning")
 def test_torch_distributed_trial_invalid_argument() -> None:
     with pytest.raises(ValueError):
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             TorchDistributedTrial(None)
         else:
             study = optuna.create_study()
@@ -64,7 +63,7 @@ def test_torch_distributed_trial_invalid_argument() -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_float(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -82,7 +81,7 @@ def test_suggest_float(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_uniform(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -100,7 +99,7 @@ def test_suggest_uniform(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_loguniform(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -118,7 +117,7 @@ def test_suggest_loguniform(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_discrete_uniform(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -136,7 +135,7 @@ def test_suggest_discrete_uniform(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_int(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -153,7 +152,7 @@ def test_suggest_int(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_categorical(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -171,7 +170,7 @@ def test_suggest_categorical(storage_mode: str) -> None:
 def test_report(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
         study: Optional[optuna.study.Study] = None
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -179,7 +178,7 @@ def test_report(storage_mode: str) -> None:
 
         trial.report(1, 0)
 
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             assert study is not None
             study.trials[0].intermediate_values[0] == 1
 
@@ -189,16 +188,16 @@ def test_report(storage_mode: str) -> None:
 def test_report_nan(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
         study: Optional[optuna.study.Study] = None
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
             trial = TorchDistributedTrial(None)
 
         with pytest.raises(TypeError):
-            trial.report("abc", 0)  # type: ignore
+            trial.report("abc", 0)  # type: ignore[arg-type]
 
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             assert study is not None
             assert len(study.trials[0].intermediate_values) == 0
 
@@ -208,7 +207,7 @@ def test_report_nan(storage_mode: str) -> None:
 @pytest.mark.parametrize("is_pruning", [False, True])
 def test_should_prune(storage_mode: str, is_pruning: bool) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage, pruner=DeterministicPruner(is_pruning))
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -222,7 +221,7 @@ def test_should_prune(storage_mode: str, is_pruning: bool) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_user_attrs(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -238,7 +237,7 @@ def test_user_attrs(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::optuna.exceptions.ExperimentalWarning")
 def test_user_attrs_with_exception() -> None:
     with StorageSupplier("sqlite") as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -252,7 +251,7 @@ def test_user_attrs_with_exception() -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_number(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -265,7 +264,7 @@ def test_number(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_datetime_start(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -278,7 +277,7 @@ def test_datetime_start(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_params(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -298,7 +297,7 @@ def test_params(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_distributions(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -325,7 +324,7 @@ def test_distributions(storage_mode: str) -> None:
 def test_updates_properties(storage_mode: str) -> None:
     """Check for any distributed deadlock following a property read."""
     with StorageSupplier(storage_mode) as storage:
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             study = optuna.create_study(storage=storage)
             trial = TorchDistributedTrial(study.ask())
         else:
@@ -342,13 +341,30 @@ def test_updates_properties(storage_mode: str) -> None:
         ]
 
         # Rank 0 can read properties without deadlock.
-        if dist.get_rank() == 0:  # type: ignore
+        if dist.get_rank() == 0:
             [getattr(trial, p) for p in property_names]
 
-        dist.barrier()  # type: ignore
+        dist.barrier()  # type: ignore[no-untyped-call]
 
         # Same with rank 1.
-        if dist.get_rank() == 1:  # type: ignore
+        if dist.get_rank() == 1:
             [getattr(trial, p) for p in property_names]
 
-        dist.barrier()  # type: ignore
+        dist.barrier()  # type: ignore[no-untyped-call]
+
+
+@pytest.mark.filterwarnings("ignore::optuna.exceptions.ExperimentalWarning")
+@pytest.mark.parametrize("storage_mode", STORAGE_MODES)
+def test_pass_frozen_trial_to_torch_distributed(storage_mode: str) -> None:
+    # Regression test of #4697
+    def objective(trial: optuna.trial.BaseTrial) -> float:
+        trial = optuna.integration.TorchDistributedTrial(trial if dist.get_rank() == 0 else None)
+        x = trial.suggest_float("x", low=-100, high=100)
+        return x * x
+
+    with StorageSupplier(storage_mode) as storage:
+        study = optuna.create_study(direction="minimize", storage=storage)
+        study.optimize(objective, n_trials=1)
+        best_trial = study.best_trial
+
+        objective(best_trial)
