@@ -2,6 +2,8 @@ from typing import Any
 from typing import cast
 from typing import Dict
 from typing import List
+from typing import Mapping
+from typing import Sequence
 from typing import Union
 
 import numpy as np
@@ -17,8 +19,10 @@ from optuna.testing.storages import StorageSupplier
 with try_import() as _imports:
     import pytorch_lightning as pl
     from pytorch_lightning import LightningModule
+    from pytorch_lightning.strategies.ddp_spawn import DDPSpawnStrategy
     import torch
     from torch import nn
+    from torch.multiprocessing.spawn import ProcessRaisedException
     import torch.nn.functional as F
 
 if not _imports.is_successful():
@@ -29,7 +33,6 @@ pytestmark = pytest.mark.integration
 
 class Model(LightningModule):
     def __init__(self) -> None:
-
         super().__init__()
         self._model = nn.Sequential(nn.Linear(4, 8))
 
@@ -37,18 +40,16 @@ class Model(LightningModule):
         return self._model(data)
 
     def training_step(
-        self, batch: List["torch.Tensor"], batch_nb: int
+        self, batch: Sequence["torch.Tensor"], batch_nb: int
     ) -> Dict[str, "torch.Tensor"]:
-
         data, target = batch
         output = self.forward(data)
         loss = F.nll_loss(output, target)
         return {"loss": loss}
 
     def validation_step(
-        self, batch: List["torch.Tensor"], batch_nb: int
+        self, batch: Sequence["torch.Tensor"], batch_nb: int
     ) -> Dict[str, "torch.Tensor"]:
-
         data, target = batch
         output = self.forward(data)
         pred = output.argmax(dim=1, keepdim=True)
@@ -58,8 +59,8 @@ class Model(LightningModule):
     def validation_epoch_end(
         self,
         outputs: Union[
-            List[Union["torch.Tensor", Dict[str, Any]]],
-            List[List[Union["torch.Tensor", Dict[str, Any]]]],
+            Sequence[Union["torch.Tensor", Mapping[str, Any]]],
+            Sequence[Sequence[Union["torch.Tensor", Mapping[str, Any]]]],
         ],
     ) -> None:
         if not len(outputs):
@@ -71,19 +72,15 @@ class Model(LightningModule):
         self.log("accuracy", accuracy)
 
     def configure_optimizers(self) -> "torch.optim.Optimizer":
-
         return torch.optim.SGD(self._model.parameters(), lr=1e-2)
 
     def train_dataloader(self) -> "torch.utils.data.DataLoader":
-
         return self._generate_dummy_dataset()
 
     def val_dataloader(self) -> "torch.utils.data.DataLoader":
-
         return self._generate_dummy_dataset()
 
     def _generate_dummy_dataset(self) -> "torch.utils.data.DataLoader":
-
         data = torch.zeros(3, 4, dtype=torch.float32)
         target = torch.zeros(3, dtype=torch.int64)
         dataset = torch.utils.data.TensorDataset(data, target)
@@ -92,34 +89,40 @@ class Model(LightningModule):
 
 class ModelDDP(Model):
     def __init__(self) -> None:
-
         super().__init__()
 
     def validation_step(
-        self, batch: List["torch.Tensor"], batch_nb: int
+        self, batch: Sequence["torch.Tensor"], batch_nb: int
     ) -> Dict[str, "torch.Tensor"]:
-
         data, target = batch
         output = self.forward(data)
         pred = output.argmax(dim=1, keepdim=True)
         accuracy = pred.eq(target.view_as(pred)).double().mean()
-
-        if self.local_rank == 0:
+        if self.global_rank == 0:
             accuracy = torch.tensor(0.3)
-        elif self.local_rank == 1:
+        elif self.global_rank == 1:
             accuracy = torch.tensor(0.6)
 
         self.log("accuracy", accuracy, sync_dist=True)
         return {"validation_accuracy": accuracy}
 
+    def validation_epoch_end(
+        self,
+        output: Union[
+            Sequence[Union["torch.Tensor", Mapping[str, Any]]],
+            Sequence[Sequence[Union["torch.Tensor", Mapping[str, Any]]]],
+        ],
+    ) -> None:
+        return
+
 
 def test_pytorch_lightning_pruning_callback() -> None:
     def objective(trial: optuna.trial.Trial) -> float:
-
+        callback = PyTorchLightningPruningCallback(trial, monitor="accuracy")
         trainer = pl.Trainer(
             max_epochs=2,
             enable_checkpointing=False,
-            callbacks=[PyTorchLightningPruningCallback(trial, monitor="accuracy")],
+            callbacks=[callback],
         )
 
         model = Model()
@@ -138,7 +141,6 @@ def test_pytorch_lightning_pruning_callback() -> None:
 
 
 def test_pytorch_lightning_pruning_callback_monitor_is_invalid() -> None:
-
     study = optuna.create_study(pruner=DeterministicPruner(True))
     trial = study.ask()
     callback = PyTorchLightningPruningCallback(trial, "InvalidMonitor")
@@ -159,18 +161,22 @@ def test_pytorch_lightning_pruning_callback_ddp_monitor(
     storage_mode: str,
 ) -> None:
     def objective(trial: optuna.trial.Trial) -> float:
-
+        callback = PyTorchLightningPruningCallback(trial, monitor="accuracy")
         trainer = pl.Trainer(
             max_epochs=2,
-            accelerator="ddp_cpu",
-            num_processes=2,
+            accelerator="cpu",
+            devices=2,
             enable_checkpointing=False,
-            callbacks=[PyTorchLightningPruningCallback(trial, monitor="accuracy")],
+            callbacks=[callback],
+            strategy=DDPSpawnStrategy(find_unused_parameters=False),
         )
 
         model = ModelDDP()
         trainer.fit(model)
 
+        # Evoke pruning manually.
+        callback.check_pruned()
+
         return 1.0
 
     with StorageSupplier(storage_mode) as storage:
@@ -193,21 +199,25 @@ def test_pytorch_lightning_pruning_callback_ddp_unsupported_storage() -> None:
     storage_mode = "inmemory"
 
     def objective(trial: optuna.trial.Trial) -> float:
-
+        callback = PyTorchLightningPruningCallback(trial, monitor="accuracy")
         trainer = pl.Trainer(
             max_epochs=1,
-            accelerator="ddp_cpu",
-            num_processes=2,
+            accelerator="cpu",
+            devices=2,
             enable_checkpointing=False,
-            callbacks=[PyTorchLightningPruningCallback(trial, monitor="accuracy")],
+            callbacks=[callback],
+            strategy=DDPSpawnStrategy(find_unused_parameters=False),
         )
 
         model = ModelDDP()
         trainer.fit(model)
 
+        # Evoke pruning manually.
+        callback.check_pruned()
+
         return 1.0
 
     with StorageSupplier(storage_mode) as storage:
         study = optuna.create_study(storage=storage, pruner=DeterministicPruner(True))
-        with pytest.raises(ValueError):
+        with pytest.raises(ProcessRaisedException):
             study.optimize(objective, n_trials=1)
