@@ -2,6 +2,7 @@ import copy
 from datetime import datetime
 import pickle
 import random
+from time import sleep
 from typing import Any
 from typing import Dict
 from typing import List
@@ -12,6 +13,7 @@ import numpy as np
 import pytest
 
 import optuna
+from optuna._typing import JSONSerializable
 from optuna.distributions import CategoricalDistribution
 from optuna.distributions import FloatDistribution
 from optuna.storages import _CachedStorage
@@ -29,7 +31,7 @@ from optuna.trial import TrialState
 
 ALL_STATES = list(TrialState)
 
-EXAMPLE_ATTRS = {
+EXAMPLE_ATTRS: Dict[str, JSONSerializable] = {
     "dataset": "MNIST",
     "none": None,
     "json_serializable": {"baseline_score": 0.001, "tags": ["image", "classification"]},
@@ -37,16 +39,13 @@ EXAMPLE_ATTRS = {
 
 
 def test_get_storage() -> None:
-
     assert isinstance(optuna.storages.get_storage(None), InMemoryStorage)
     assert isinstance(optuna.storages.get_storage("sqlite:///:memory:"), _CachedStorage)
 
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_create_new_study(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
 
         frozen_studies = storage.get_all_studies()
@@ -65,9 +64,7 @@ def test_create_new_study(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_create_new_study_unique_id(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         study_id2 = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         storage.delete_study(study_id2)
@@ -83,9 +80,7 @@ def test_create_new_study_unique_id(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_create_new_study_with_name(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         # Generate unique study_name from the current function name and storage_mode.
         function_name = test_create_new_study_with_name.__name__
         study_name = function_name + "/" + storage_mode
@@ -101,9 +96,7 @@ def test_create_new_study_with_name(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_delete_study(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         storage.create_new_trial(study_id)
         trials = storage.get_all_trials(study_id)
@@ -126,7 +119,6 @@ def test_delete_study(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_delete_study_after_create_multiple_studies(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id1 = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         study_id2 = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
@@ -142,9 +134,7 @@ def test_delete_study_after_create_multiple_studies(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_study_id_from_name_and_get_study_name_from_id(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         # Generate unique study_name from the current function name and storage_mode.
         function_name = test_get_study_id_from_name_and_get_study_name_from_id.__name__
         study_name = function_name + "/" + storage_mode
@@ -166,9 +156,7 @@ def test_get_study_id_from_name_and_get_study_name_from_id(storage_mode: str) ->
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_and_get_study_directions(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         for target in [
             (StudyDirection.MINIMIZE,),
             (StudyDirection.MAXIMIZE,),
@@ -198,12 +186,10 @@ def test_set_and_get_study_directions(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_and_get_study_user_attrs(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
 
         def check_set_and_get(key: str, value: Any) -> None:
-
             storage.set_study_user_attr(study_id, key, value)
             assert storage.get_study_user_attrs(study_id)[key] == value
 
@@ -227,12 +213,10 @@ def test_set_and_get_study_user_attrs(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_and_get_study_system_attrs(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
 
         def check_set_and_get(key: str, value: Any) -> None:
-
             storage.set_study_system_attr(study_id, key, value)
             assert storage.get_study_system_attrs(study_id)[key] == value
 
@@ -256,7 +240,6 @@ def test_set_and_get_study_system_attrs(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_study_user_and_system_attrs_confusion(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         for key, value in EXAMPLE_ATTRS.items():
@@ -303,12 +286,13 @@ def test_create_new_trial(storage_mode: str) -> None:
         assert all(t.value is None for t in trials)
 
     with StorageSupplier(storage_mode) as storage:
-
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         n_trial_in_study = 3
         for i in range(n_trial_in_study):
             time_before_creation = datetime.now()
+            sleep(0.001)  # Sleep 1ms to avoid faulty assertion on Windows OS.
             trial_id = storage.create_new_trial(study_id)
+            sleep(0.001)
             time_after_creation = datetime.now()
 
             trials = storage.get_all_trials(study_id)
@@ -339,7 +323,6 @@ def test_create_new_trial(storage_mode: str) -> None:
 def test_create_new_trial_with_template_trial(
     storage_mode: str, start_time: datetime, complete_time: datetime
 ) -> None:
-
     template_trial = FrozenTrial(
         state=TrialState.COMPLETE,
         value=10000,
@@ -370,7 +353,6 @@ def test_create_new_trial_with_template_trial(
         assert all(t.value == template_trial.value for t in trials)
 
     with StorageSupplier(storage_mode) as storage:
-
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
 
         n_trial_in_study = 3
@@ -396,7 +378,6 @@ def test_create_new_trial_with_template_trial(
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trial_number_from_id(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         # Check if trial_number starts from 0.
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
@@ -413,9 +394,7 @@ def test_get_trial_number_from_id(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_trial_state_values_for_state(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         trial_ids = [storage.create_new_trial(study_id) for _ in ALL_STATES]
 
@@ -457,7 +436,6 @@ def test_set_trial_state_values_for_state(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trial_param_and_get_trial_params(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         _, study_to_trials = _setup_studies(storage, n_study=2, n_trial=5, seed=1)
 
@@ -483,9 +461,7 @@ def test_get_trial_param_and_get_trial_params(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_trial_param(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         # Setup test across multiple studies and trials.
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         trial_id_1 = storage.create_new_trial(study_id)
@@ -556,9 +532,7 @@ def test_set_trial_param(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_trial_state_values_for_values(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         # Setup test across multiple studies and trials.
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         trial_id_1 = storage.create_new_trial(study_id)
@@ -600,9 +574,7 @@ def test_set_trial_state_values_for_values(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_trial_intermediate_value(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
-
         # Setup test across multiple studies and trials.
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         trial_id_1 = storage.create_new_trial(study_id)
@@ -647,7 +619,6 @@ def test_set_trial_intermediate_value(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trial_user_attrs(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         _, study_to_trials = _setup_studies(storage, n_study=2, n_trial=5, seed=10)
         assert all(
@@ -663,14 +634,12 @@ def test_get_trial_user_attrs(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_trial_user_attr(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         trial_id_1 = storage.create_new_trial(
             storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         )
 
         def check_set_and_get(trial_id: int, key: str, value: Any) -> None:
-
             storage.set_trial_user_attr(trial_id, key, value)
             assert storage.get_trial(trial_id).user_attrs[key] == value
 
@@ -703,7 +672,6 @@ def test_set_trial_user_attr(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trial_system_attrs(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         _, study_to_trials = _setup_studies(storage, n_study=2, n_trial=5, seed=10)
         assert all(
@@ -719,13 +687,11 @@ def test_get_trial_system_attrs(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_set_trial_system_attr(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
         trial_id_1 = storage.create_new_trial(study_id)
 
         def check_set_and_get(trial_id: int, key: str, value: Any) -> None:
-
             storage.set_trial_system_attr(trial_id, key, value)
             assert storage.get_trial_system_attrs(trial_id)[key] == value
 
@@ -757,7 +723,6 @@ def test_set_trial_system_attr(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_all_studies(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         expected_frozen_studies, _ = _setup_studies(storage, n_study=10, n_trial=10, seed=46)
         frozen_studies = storage.get_all_studies()
@@ -777,7 +742,6 @@ def test_get_all_studies(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trial(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         _, study_to_trials = _setup_studies(storage, n_study=2, n_trial=20, seed=47)
 
@@ -795,7 +759,6 @@ def test_get_trial(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_all_trials(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         _, study_to_trials = _setup_studies(storage, n_study=2, n_trial=20, seed=48)
 
@@ -812,7 +775,6 @@ def test_get_all_trials(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_all_trials_deepcopy_option(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         frozen_studies, study_to_trials = _setup_studies(storage, n_study=2, n_trial=5, seed=49)
 
@@ -830,7 +792,6 @@ def test_get_all_trials_deepcopy_option(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_all_trials_state_option(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id = storage.create_new_study(directions=[StudyDirection.MAXIMIZE])
         generator = random.Random(51)
@@ -870,7 +831,6 @@ def test_get_all_trials_state_option(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_n_trials(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id_to_frozen_studies, _ = _setup_studies(storage, n_study=2, n_trial=7, seed=50)
         for study_id in study_id_to_frozen_studies:
@@ -883,7 +843,6 @@ def test_get_n_trials(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_n_trials_state_option(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id = storage.create_new_study(directions=(StudyDirection.MAXIMIZE,))
         generator = random.Random(51)
@@ -910,10 +869,21 @@ def test_get_n_trials_state_option(storage_mode: str) -> None:
 
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
-def test_get_best_trial(storage_mode: str) -> None:
-
+@pytest.mark.parametrize("direction", [StudyDirection.MAXIMIZE, StudyDirection.MINIMIZE])
+@pytest.mark.parametrize(
+    "values",
+    [
+        [0.0, 1.0, 2.0],
+        [0.0, float("inf"), 1.0],
+        [0.0, float("-inf"), 1.0],
+        [float("inf"), 0.0, 1.0, float("-inf")],
+        [float("inf")],
+        [float("-inf")],
+    ],
+)
+def test_get_best_trial(storage_mode: str, direction: StudyDirection, values: List[float]) -> None:
     with StorageSupplier(storage_mode) as storage:
-        study_id = storage.create_new_study(directions=[StudyDirection.MAXIMIZE])
+        study_id = storage.create_new_study(directions=[direction])
         with pytest.raises(ValueError):
             storage.get_best_trial(study_id)
 
@@ -921,12 +891,14 @@ def test_get_best_trial(storage_mode: str) -> None:
             storage.get_best_trial(study_id + 1)
 
         generator = random.Random(51)
-        for i in range(3):
+
+        for v in values:
             template_trial = _generate_trial(generator)
             template_trial.state = TrialState.COMPLETE
-            template_trial.value = float(i)
+            template_trial.value = v
             storage.create_new_trial(study_id, template_trial=template_trial)
-        assert storage.get_best_trial(study_id).number == i
+        expected_value = max(values) if direction == StudyDirection.MAXIMIZE else min(values)
+        assert storage.get_best_trial(study_id).value == expected_value
 
 
 def test_get_trials_excluded_trial_ids() -> None:
@@ -1002,7 +974,7 @@ def _generate_trial(generator: random.Random) -> FrozenTrial:
     params = {}
     distributions = {}
     user_attrs = {}
-    system_attrs = {}
+    system_attrs: Dict[str, Any] = {}
     intermediate_values = {}
     for key, (value, dist) in example_params.items():
         if generator.choice([True, False]):
@@ -1033,7 +1005,6 @@ def _generate_trial(generator: random.Random) -> FrozenTrial:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_best_trial_for_multi_objective_optimization(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study_id = storage.create_new_study(
             directions=(StudyDirection.MAXIMIZE, StudyDirection.MINIMIZE)
@@ -1052,7 +1023,6 @@ def test_get_best_trial_for_multi_objective_optimization(storage_mode: str) -> N
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trial_id_from_study_id_trial_number(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         with pytest.raises(KeyError):  # Matching study does not exist.
             storage.get_trial_id_from_study_id_trial_number(study_id=0, trial_number=0)
