@@ -3,6 +3,7 @@ from concurrent.futures import ThreadPoolExecutor
 import copy
 import multiprocessing
 import pickle
+import platform
 import threading
 import time
 from typing import Any
@@ -30,7 +31,9 @@ from optuna import Study
 from optuna import Trial
 from optuna import TrialPruned
 from optuna.exceptions import DuplicatedStudyError
+from optuna.exceptions import ExperimentalWarning
 from optuna.study import StudyDirection
+from optuna.study.study import _SYSTEM_ATTR_METRIC_NAMES
 from optuna.testing.objectives import fail_objective
 from optuna.testing.storages import STORAGE_MODES
 from optuna.testing.storages import StorageSupplier
@@ -42,7 +45,6 @@ CallbackFuncType = Callable[[Study, FrozenTrial], None]
 
 
 def func(trial: Trial) -> float:
-
     x = trial.suggest_float("x", -10.0, 10.0)
     y = trial.suggest_float("y", 20, 30, log=True)
     z = trial.suggest_categorical("z", (-1.0, 1.0))
@@ -51,13 +53,11 @@ def func(trial: Trial) -> float:
 
 class Func:
     def __init__(self, sleep_sec: Optional[float] = None) -> None:
-
         self.n_calls = 0
         self.sleep_sec = sleep_sec
         self.lock = threading.Lock()
 
     def __call__(self, trial: Trial) -> float:
-
         with self.lock:
             self.n_calls += 1
 
@@ -71,25 +71,21 @@ class Func:
 
 
 def check_params(params: Dict[str, Any]) -> None:
-
     assert sorted(params.keys()) == ["x", "y", "z"]
 
 
 def check_value(value: Optional[float]) -> None:
-
     assert isinstance(value, float)
     assert -1.0 <= value <= 12.0**2 + 5.0**2 + 1.0
 
 
 def check_frozen_trial(frozen_trial: FrozenTrial) -> None:
-
     if frozen_trial.state == TrialState.COMPLETE:
         check_params(frozen_trial.params)
         check_value(frozen_trial.value)
 
 
 def check_study(study: Study) -> None:
-
     for trial in study.trials:
         check_frozen_trial(trial)
 
@@ -120,14 +116,12 @@ def stop_objective(threshold_number: int) -> Callable[[Trial], float]:
 
 
 def test_optimize_trivial_in_memory_new() -> None:
-
     study = create_study()
     study.optimize(func, n_trials=10)
     check_study(study)
 
 
 def test_optimize_trivial_in_memory_resume() -> None:
-
     study = create_study()
     study.optimize(func, n_trials=10)
     study.optimize(func, n_trials=10)
@@ -135,14 +129,12 @@ def test_optimize_trivial_in_memory_resume() -> None:
 
 
 def test_optimize_trivial_rdb_resume_study() -> None:
-
     study = create_study(storage="sqlite:///:memory:")
     study.optimize(func, n_trials=10)
     check_study(study)
 
 
 def test_optimize_with_direction() -> None:
-
     study = create_study(direction="minimize")
     study.optimize(func, n_trials=10)
     assert study.direction == StudyDirection.MINIMIZE
@@ -161,7 +153,6 @@ def test_optimize_with_direction() -> None:
 @pytest.mark.parametrize("n_jobs", (1, 2, -1))
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_optimize_parallel(n_trials: int, n_jobs: int, storage_mode: str) -> None:
-
     f = Func()
 
     with StorageSupplier(storage_mode) as storage:
@@ -186,7 +177,6 @@ def test_optimize_with_thread_pool_executor() -> None:
 @pytest.mark.parametrize("n_jobs", (1, 2, -1))
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_optimize_parallel_timeout(n_trials: int, n_jobs: int, storage_mode: str) -> None:
-
     sleep_sec = 0.1
     timeout_sec = 1.0
     f = Func(sleep_sec=sleep_sec)
@@ -210,7 +200,6 @@ def test_optimize_parallel_timeout(n_trials: int, n_jobs: int, storage_mode: str
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_optimize_with_catch(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
 
@@ -234,14 +223,12 @@ def test_optimize_with_catch(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("catch", [ValueError, (ValueError,), [ValueError], {ValueError}])
 def test_optimize_with_catch_valid_type(catch: Any) -> None:
-
     study = create_study()
     study.optimize(fail_objective, n_trials=20, catch=catch)
 
 
 @pytest.mark.parametrize("catch", [None, 1])
 def test_optimize_with_catch_invalid_type(catch: Any) -> None:
-
     study = create_study()
 
     with pytest.raises(TypeError):
@@ -251,7 +238,6 @@ def test_optimize_with_catch_invalid_type(catch: Any) -> None:
 @pytest.mark.parametrize("n_jobs", (2, -1))
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_optimize_with_reseeding(n_jobs: int, storage_mode: str) -> None:
-
     f = Func()
 
     with StorageSupplier(storage_mode) as storage:
@@ -279,7 +265,6 @@ def test_call_another_study_optimize_in_optimize() -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_study_set_and_get_user_attrs(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
 
@@ -290,7 +275,6 @@ def test_study_set_and_get_user_attrs(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_trial_set_and_get_user_attrs(storage_mode: str) -> None:
     def f(trial: Trial) -> float:
-
         trial.set_user_attr("train_accuracy", 1)
         assert trial.user_attrs["train_accuracy"] == 1
         return 0.0
@@ -305,7 +289,6 @@ def test_trial_set_and_get_user_attrs(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 @pytest.mark.parametrize("include_best_trial", [True, False])
 def test_get_all_study_summaries(storage_mode: str, include_best_trial: bool) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         study.optimize(func, n_trials=5)
@@ -323,7 +306,6 @@ def test_get_all_study_summaries(storage_mode: str, include_best_trial: bool) ->
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_all_study_summaries_with_no_trials(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
 
@@ -336,7 +318,6 @@ def test_get_all_study_summaries_with_no_trials(storage_mode: str) -> None:
 
 
 def test_study_pickle() -> None:
-
     study_1 = create_study()
     study_1.optimize(func, n_trials=10)
     check_study(study_1)
@@ -354,7 +335,6 @@ def test_study_pickle() -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_create_study(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         # Test creating a new study.
         study = create_study(storage=storage, load_if_exists=False)
@@ -368,7 +348,6 @@ def test_create_study(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_load_study(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         if storage is None:
             # `InMemoryStorage` can not be used with `load_study` function.
@@ -390,7 +369,6 @@ def test_load_study(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_load_study_study_name_none(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         if storage is None:
             # `InMemoryStorage` can not be used with `load_study` function.
@@ -415,7 +393,6 @@ def test_load_study_study_name_none(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_delete_study(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         # Test deleting a non-existing study.
         with pytest.raises(KeyError):
@@ -488,7 +465,6 @@ def test_copy_study_to_study_name(from_storage_mode: str, to_storage_mode: str)
 
 def test_nested_optimization() -> None:
     def objective(trial: Trial) -> float:
-
         with pytest.raises(RuntimeError):
             trial.study.optimize(lambda _: 0.0, n_trials=1)
 
@@ -499,7 +475,6 @@ def test_nested_optimization() -> None:
 
 
 def test_stop_in_objective() -> None:
-
     # Test stopping the optimization: it should stop once the trial number reaches 4.
     study = create_study()
     study.optimize(stop_objective(4), n_trials=10)
@@ -543,7 +518,6 @@ def test_stop_outside_optimize() -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_add_trial(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
@@ -555,9 +529,20 @@ def test_add_trial(storage_mode: str) -> None:
         assert study.best_value == 0.8
 
 
+def test_add_trial_invalid_values_length() -> None:
+    study = create_study()
+    trial = create_trial(values=[0, 0])
+    with pytest.raises(ValueError):
+        study.add_trial(trial)
+
+    study = create_study(directions=["minimize", "minimize"])
+    trial = create_trial(value=0)
+    with pytest.raises(ValueError):
+        study.add_trial(trial)
+
+
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_add_trials(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
@@ -582,7 +567,6 @@ def test_add_trials(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_enqueue_trial_properly_sets_param_values(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
@@ -591,7 +575,6 @@ def test_enqueue_trial_properly_sets_param_values(storage_mode: str) -> None:
         study.enqueue_trial(params={"x": -1, "y": 0})
 
         def objective(trial: Trial) -> float:
-
             x = trial.suggest_int("x", -10, 10)
             y = trial.suggest_int("y", -10, 10)
             return x**2 + y**2
@@ -608,7 +591,6 @@ def test_enqueue_trial_properly_sets_param_values(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_enqueue_trial_with_unfixed_parameters(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
@@ -616,7 +598,6 @@ def test_enqueue_trial_with_unfixed_parameters(storage_mode: str) -> None:
         study.enqueue_trial(params={"x": -5})
 
         def objective(trial: Trial) -> float:
-
             x = trial.suggest_int("x", -10, 10)
             y = trial.suggest_int("y", -10, 10)
             return x**2 + y**2
@@ -629,7 +610,6 @@ def test_enqueue_trial_with_unfixed_parameters(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_enqueue_trial_properly_sets_user_attr(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
@@ -638,7 +618,6 @@ def test_enqueue_trial_properly_sets_user_attr(storage_mode: str) -> None:
         study.enqueue_trial(params={"x": 0, "y": 0}, user_attrs={"is_optimal": True})
 
         def objective(trial: Trial) -> float:
-
             x = trial.suggest_int("x", -10, 10)
             y = trial.suggest_int("y", -10, 10)
             return x**2 + y**2
@@ -662,7 +641,6 @@ def test_enqueue_trial_with_out_of_range_parameters(storage_mode: str) -> None:
         study.enqueue_trial(params={"x": fixed_value})
 
         def objective(trial: Trial) -> float:
-
             return trial.suggest_int("x", -10, 10)
 
         with pytest.warns(UserWarning):
@@ -679,7 +657,6 @@ def test_enqueue_trial_with_out_of_range_parameters(storage_mode: str) -> None:
         study.enqueue_trial(params={"x": fixed_value})
 
         def objective(trial: Trial) -> float:
-
             return trial.suggest_int("x", 1, 1)  # Single element.
 
         with pytest.warns(UserWarning):
@@ -690,13 +667,11 @@ def test_enqueue_trial_with_out_of_range_parameters(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_enqueue_trial_skips_existing_finished(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
 
         def objective(trial: Trial) -> float:
-
             x = trial.suggest_int("x", -10, 10)
             y = trial.suggest_int("y", -10, 10)
             return x**2 + y**2
@@ -716,13 +691,11 @@ def test_enqueue_trial_skips_existing_finished(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_enqueue_trial_skips_existing_waiting(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
 
         def objective(trial: Trial) -> float:
-
             x = trial.suggest_int("x", -10, 10)
             y = trial.suggest_int("y", -10, 10)
             return x**2 + y**2
@@ -746,13 +719,11 @@ def test_enqueue_trial_skips_existing_waiting(storage_mode: str) -> None:
 def test_enqueue_trial_skip_existing_allows_unfixed(
     storage_mode: str, new_params: Dict[str, int]
 ) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         assert len(study.trials) == 0
 
         def objective(trial: Trial) -> float:
-
             x = trial.suggest_int("x", -10, 10)
             y = trial.suggest_int("y", -10, 10)
             if trial.number == 1:
@@ -780,7 +751,6 @@ def test_enqueue_trial_skip_existing_allows_unfixed(
     "param", ["foo", 1, 1.1, 1e17, 1e-17, float("inf"), float("-inf"), float("nan"), None]
 )
 def test_enqueue_trial_skip_existing_handles_common_types(storage_mode: str, param: Any) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         study.enqueue_trial({"x": param})
@@ -792,7 +762,6 @@ def test_enqueue_trial_skip_existing_handles_common_types(storage_mode: str, par
 
 @patch("optuna.study._optimize.gc.collect")
 def test_optimize_with_gc(collect_mock: Mock) -> None:
-
     study = create_study()
     study.optimize(func, n_trials=10, gc_after_trial=True)
     check_study(study)
@@ -801,7 +770,6 @@ def test_optimize_with_gc(collect_mock: Mock) -> None:
 
 @patch("optuna.study._optimize.gc.collect")
 def test_optimize_without_gc(collect_mock: Mock) -> None:
-
     study = create_study()
     study.optimize(func, n_trials=10, gc_after_trial=False)
     check_study(study)
@@ -810,7 +778,6 @@ def test_optimize_without_gc(collect_mock: Mock) -> None:
 
 @pytest.mark.parametrize("n_jobs", [1, 2])
 def test_optimize_with_progbar(n_jobs: int, capsys: _pytest.capture.CaptureFixture) -> None:
-
     study = create_study()
     study.optimize(lambda _: 1.0, n_trials=10, n_jobs=n_jobs, show_progress_bar=True)
     _, err = capsys.readouterr()
@@ -819,12 +786,13 @@ def test_optimize_with_progbar(n_jobs: int, capsys: _pytest.capture.CaptureFixtu
     assert "Best trial: 0" in err
     assert "Best value: 1" in err
     assert "10/10" in err
-    assert "100%" in err
+    if platform.system() != "Windows":
+        # Skip this assertion because the progress bar sometimes stops at 99% on Windows.
+        assert "100%" in err
 
 
 @pytest.mark.parametrize("n_jobs", [1, 2])
 def test_optimize_without_progbar(n_jobs: int, capsys: _pytest.capture.CaptureFixture) -> None:
-
     study = create_study()
     study.optimize(lambda _: 1.0, n_trials=10, n_jobs=n_jobs)
     _, err = capsys.readouterr()
@@ -832,11 +800,12 @@ def test_optimize_without_progbar(n_jobs: int, capsys: _pytest.capture.CaptureFi
     assert "Best trial: 0" not in err
     assert "Best value: 1" not in err
     assert "10/10" not in err
-    assert "100%" not in err
+    if platform.system() != "Windows":
+        # Skip this assertion because the progress bar sometimes stops at 99% on Windows.
+        assert "100%" not in err
 
 
 def test_optimize_with_progbar_timeout(capsys: _pytest.capture.CaptureFixture) -> None:
-
     study = create_study()
     study.optimize(lambda _: 1.0, timeout=2.0, show_progress_bar=True)
     _, err = capsys.readouterr()
@@ -844,11 +813,12 @@ def test_optimize_with_progbar_timeout(capsys: _pytest.capture.CaptureFixture) -
     assert "Best trial: 0" in err
     assert "Best value: 1" in err
     assert "00:02/00:02" in err
-    assert "100%" in err
+    if platform.system() != "Windows":
+        # Skip this assertion because the progress bar sometimes stops at 99% on Windows.
+        assert "100%" in err
 
 
 def test_optimize_with_progbar_parallel_timeout(capsys: _pytest.capture.CaptureFixture) -> None:
-
     study = create_study()
     with pytest.warns(
         UserWarning, match="The timeout-based progress bar is not supported with n_jobs != 1."
@@ -883,7 +853,6 @@ def test_optimize_with_progbar_timeout_formats(
 def test_optimize_without_progbar_timeout(
     n_jobs: int, capsys: _pytest.capture.CaptureFixture
 ) -> None:
-
     study = create_study()
     study.optimize(lambda _: 1.0, timeout=2.0, n_jobs=n_jobs)
     _, err = capsys.readouterr()
@@ -891,14 +860,15 @@ def test_optimize_without_progbar_timeout(
     assert "Best trial: 0" not in err
     assert "Best value: 1.0" not in err
     assert "00:02/00:02" not in err
-    assert "100%" not in err
+    if platform.system() != "Windows":
+        # Skip this assertion because the progress bar sometimes stops at 99% on Windows.
+        assert "100%" not in err
 
 
 @pytest.mark.parametrize("n_jobs", [1, 2])
 def test_optimize_progbar_n_trials_prioritized(
     n_jobs: int, capsys: _pytest.capture.CaptureFixture
 ) -> None:
-
     study = create_study()
     study.optimize(lambda _: 1.0, n_trials=10, n_jobs=n_jobs, timeout=10.0, show_progress_bar=True)
     _, err = capsys.readouterr()
@@ -906,7 +876,9 @@ def test_optimize_progbar_n_trials_prioritized(
     assert "Best trial: 0" in err
     assert "Best value: 1" in err
     assert "10/10" in err
-    assert "100%" in err
+    if platform.system() != "Windows":
+        # Skip this assertion because the progress bar sometimes stops at 99% on Windows.
+        assert "100%" in err
     assert "it" in err
 
 
@@ -914,7 +886,6 @@ def test_optimize_progbar_n_trials_prioritized(
 def test_optimize_without_progbar_n_trials_prioritized(
     n_jobs: int, capsys: _pytest.capture.CaptureFixture
 ) -> None:
-
     study = create_study()
     study.optimize(lambda _: 1.0, n_trials=10, n_jobs=n_jobs, timeout=10.0)
     _, err = capsys.readouterr()
@@ -951,12 +922,10 @@ def test_optimize_without_progbar_no_constraints(
 
 @pytest.mark.parametrize("n_jobs", [1, 4])
 def test_callbacks(n_jobs: int) -> None:
-
     lock = threading.Lock()
 
     def with_lock(f: CallbackFuncType) -> CallbackFuncType:
         def callback(study: Study, trial: FrozenTrial) -> None:
-
             with lock:
                 f(study, trial)
 
@@ -965,7 +934,6 @@ def test_callbacks(n_jobs: int) -> None:
     study = create_study()
 
     def objective(trial: Trial) -> float:
-
         return trial.suggest_int("x", 1, 1)
 
     # Empty callback list.
@@ -1012,7 +980,6 @@ def test_callbacks(n_jobs: int) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trials(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
         study.optimize(lambda t: t.suggest_int("x", 1, 5), n_trials=5)
@@ -1035,7 +1002,6 @@ def test_get_trials(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_trials_state_option(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
 
@@ -1074,7 +1040,6 @@ def test_get_trials_state_option(storage_mode: str) -> None:
 
 
 def test_log_completed_trial(capsys: _pytest.capture.CaptureFixture) -> None:
-
     # We need to reconstruct our default handler to properly capture stderr.
     logging._reset_library_root_logger()
     logging.set_verbosity(logging.INFO)
@@ -1096,7 +1061,6 @@ def test_log_completed_trial(capsys: _pytest.capture.CaptureFixture) -> None:
 
 
 def test_log_completed_trial_skip_storage_access() -> None:
-
     study = create_study()
 
     # Create a trial to retrieve it as the `study.best_trial`.
@@ -1237,7 +1201,6 @@ def test_ask_distribution_conversion() -> None:
         FutureWarning,
         match="See https://github.com/optuna/optuna/issues/2941",
     ) as record:
-
         trial = study.ask(fixed_distributions=fixed_distributions)
         assert len(record) == 6
 
@@ -1498,7 +1461,6 @@ def test_tell_duplicate_tell() -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_enqueued_trial_datetime_start(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage)
 
@@ -1516,7 +1478,6 @@ def test_enqueued_trial_datetime_start(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_study_summary_datetime_start_calculation(storage_mode: str) -> None:
-
     with StorageSupplier(storage_mode) as storage:
 
         def objective(trial: Trial) -> float:
@@ -1543,7 +1504,6 @@ def _process_tell(study: Study, trial: Union[Trial, int], values: float) -> None
 
 
 def test_tell_from_another_process() -> None:
-
     pool = multiprocessing.Pool()
 
     with StorageSupplier("sqlite") as storage:
@@ -1567,7 +1527,7 @@ def test_tell_from_another_process() -> None:
         assert study.best_value == 1.2
 
         # Should fail because the trial0 is already finished.
-        with pytest.raises(RuntimeError):
+        with pytest.raises(ValueError):
             pool.starmap(_process_tell, [(study, trial0, 1.2)])
 
 
@@ -1592,3 +1552,28 @@ def test_pop_waiting_trial_thread_safe(storage_mode: str) -> None:
             for future in as_completed(futures):
                 trial_id_set.add(future.result())
         assert len(trial_id_set) == num_enqueued
+
+
+def test_set_metric_names() -> None:
+    metric_names = ["v0", "v1"]
+    study = create_study(directions=["minimize", "minimize"])
+    study.set_metric_names(metric_names)
+
+    got_metric_names = study._storage.get_study_system_attrs(study._study_id).get(
+        _SYSTEM_ATTR_METRIC_NAMES
+    )
+    assert got_metric_names is not None
+    assert metric_names == got_metric_names
+
+
+def test_set_metric_names_experimental_warning() -> None:
+    study = create_study()
+    with pytest.warns(ExperimentalWarning):
+        study.set_metric_names(["v0"])
+
+
+def test_set_invalid_metric_names() -> None:
+    metric_names = ["v0", "v1", "v2"]
+    study = create_study(directions=["minimize", "minimize"])
+    with pytest.raises(ValueError):
+        study.set_metric_names(metric_names)
