@@ -1,6 +1,5 @@
 import datetime
 import math
-import tempfile
 from typing import Any
 from typing import Dict
 from typing import List
@@ -26,13 +25,14 @@ from optuna.testing.pruners import DeterministicPruner
 from optuna.testing.samplers import DeterministicSampler
 from optuna.testing.storages import STORAGE_MODES
 from optuna.testing.storages import StorageSupplier
+from optuna.testing.tempfile_pool import NamedTemporaryFilePool
 from optuna.trial import Trial
+from optuna.trial._trial import _LazyTrialSystemAttrs
 
 
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_check_distribution_suggest_float(storage_mode: str) -> None:
-
     sampler = samplers.RandomSampler()
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage, sampler=sampler)
@@ -66,7 +66,6 @@ def test_check_distribution_suggest_float(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_check_distribution_suggest_uniform(storage_mode: str) -> None:
-
     sampler = samplers.RandomSampler()
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage, sampler=sampler)
@@ -91,7 +90,6 @@ def test_check_distribution_suggest_uniform(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_check_distribution_suggest_loguniform(storage_mode: str) -> None:
-
     sampler = samplers.RandomSampler()
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage, sampler=sampler)
@@ -116,7 +114,6 @@ def test_check_distribution_suggest_loguniform(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_check_distribution_suggest_discrete_uniform(storage_mode: str) -> None:
-
     sampler = samplers.RandomSampler()
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage, sampler=sampler)
@@ -141,7 +138,6 @@ def test_check_distribution_suggest_discrete_uniform(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 @pytest.mark.parametrize("enable_log", [False, True])
 def test_check_distribution_suggest_int(storage_mode: str, enable_log: bool) -> None:
-
     sampler = samplers.RandomSampler()
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage, sampler=sampler)
@@ -165,7 +161,6 @@ def test_check_distribution_suggest_int(storage_mode: str, enable_log: bool) ->
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_check_distribution_suggest_categorical(storage_mode: str) -> None:
-
     sampler = samplers.RandomSampler()
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage, sampler=sampler)
@@ -187,7 +182,6 @@ def test_check_distribution_suggest_categorical(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_uniform(storage_mode: str) -> None:
-
     sampler = DeterministicSampler({"x": 1.0, "y": 2.0})
 
     with StorageSupplier(storage_mode) as storage:
@@ -203,7 +197,6 @@ def test_suggest_uniform(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_loguniform(storage_mode: str) -> None:
-
     with pytest.raises(ValueError):
         FloatDistribution(low=1.0, high=0.9, log=True)
 
@@ -225,7 +218,6 @@ def test_suggest_loguniform(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_discrete_uniform(storage_mode: str) -> None:
-
     sampler = DeterministicSampler({"x": 1.0, "y": 2.0})
 
     with StorageSupplier(storage_mode) as storage:
@@ -247,11 +239,9 @@ def test_suggest_discrete_uniform(storage_mode: str) -> None:
 @pytest.mark.filterwarnings("ignore::FutureWarning")
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_low_equals_high(storage_mode: str) -> None:
-
     with patch.object(
         distributions, "_get_single_value", wraps=distributions._get_single_value
     ) as mock_object, StorageSupplier(storage_mode) as storage:
-
         study = create_study(storage=storage, sampler=samplers.TPESampler(n_startup_trials=0))
 
         trial = Trial(study, study._storage.create_new_trial(study._study_id))
@@ -316,7 +306,6 @@ def test_suggest_low_equals_high(storage_mode: str) -> None:
     ],
 )
 def test_suggest_discrete_uniform_range(storage_mode: str, range_config: Dict[str, float]) -> None:
-
     sampler = samplers.RandomSampler()
 
     # Check upper endpoints.
@@ -353,7 +342,6 @@ def test_suggest_discrete_uniform_range(storage_mode: str, range_config: Dict[st
 
 
 def test_suggest_float_invalid_step() -> None:
-
     study = create_study()
     trial = study.ask()
 
@@ -366,7 +354,6 @@ def test_suggest_float_invalid_step() -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_int(storage_mode: str) -> None:
-
     sampler = DeterministicSampler({"x": 1, "y": 2})
 
     with StorageSupplier(storage_mode) as storage:
@@ -389,7 +376,6 @@ def test_suggest_int(storage_mode: str) -> None:
     ],
 )
 def test_suggest_int_range(storage_mode: str, range_config: Dict[str, int]) -> None:
-
     sampler = samplers.RandomSampler()
 
     # Check upper endpoints.
@@ -414,7 +400,6 @@ def test_suggest_int_range(storage_mode: str, range_config: Dict[str, int]) -> N
     with patch.object(sampler, "sample_independent", mock) as mock_object, StorageSupplier(
         storage_mode
     ) as storage:
-
         study = create_study(storage=storage, sampler=sampler)
         trial = Trial(study, study._storage.create_new_trial(study._study_id))
 
@@ -427,7 +412,6 @@ def test_suggest_int_range(storage_mode: str, range_config: Dict[str, int]) -> N
 
 
 def test_suggest_int_invalid_step() -> None:
-
     study = create_study()
     trial = study.ask()
 
@@ -440,7 +424,6 @@ def test_suggest_int_invalid_step() -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_int_log(storage_mode: str) -> None:
-
     sampler = DeterministicSampler({"x": 1, "y": 2})
 
     with StorageSupplier(storage_mode) as storage:
@@ -455,7 +438,6 @@ def test_suggest_int_log(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_suggest_int_log_invalid_range(storage_mode: str) -> None:
-
     sampler = samplers.RandomSampler()
     with StorageSupplier(storage_mode) as storage:
         study = create_study(storage=storage, sampler=sampler)
@@ -476,7 +458,6 @@ def test_suggest_int_log_invalid_range(storage_mode: str) -> None:
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_distributions(storage_mode: str) -> None:
     def objective(trial: Trial) -> float:
-
         trial.suggest_float("a", 0, 10)
         trial.suggest_float("b", 0.1, 10, log=True)
         trial.suggest_float("c", 0, 10, step=1)
@@ -501,7 +482,6 @@ def test_distributions(storage_mode: str) -> None:
 
 
 def test_should_prune() -> None:
-
     pruner = DeterministicPruner(True)
     study = create_study(pruner=pruner)
     trial = Trial(study, study._storage.create_new_trial(study._study_id))
@@ -542,7 +522,6 @@ def test_relative_parameters(storage_mode: str) -> None:
         study = create_study(storage=storage, sampler=sampler)
 
         def create_trial() -> Trial:
-
             return Trial(study, study._storage.create_new_trial(study._study_id))
 
         # Suggested by `sample_relative`.
@@ -581,11 +560,9 @@ def test_relative_parameters(storage_mode: str) -> None:
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_datetime_start(storage_mode: str) -> None:
-
     trial_datetime_start: List[Optional[datetime.datetime]] = [None]
 
     def objective(trial: Trial) -> float:
-
         trial_datetime_start[0] = trial.datetime_start
         return 1.0
 
@@ -597,7 +574,6 @@ def test_datetime_start(storage_mode: str) -> None:
 
 
 def test_report() -> None:
-
     study = create_study()
     trial = Trial(study, study._storage.create_new_trial(study._study_id))
 
@@ -627,7 +603,6 @@ def test_report() -> None:
 
 
 def test_report_warning() -> None:
-
     study = create_study()
     trial = study.ask()
 
@@ -639,7 +614,6 @@ def test_report_warning() -> None:
 
 
 def test_study_id() -> None:
-
     study = create_study()
     trial = Trial(study, study._storage.create_new_trial(study._study_id))
 
@@ -691,7 +665,7 @@ def test_raise_error_for_should_prune_multi_objectives() -> None:
 def test_persisted_param() -> None:
     study_name = "my_study"
 
-    with tempfile.NamedTemporaryFile() as fp:
+    with NamedTemporaryFilePool() as fp:
         storage = f"sqlite:///{fp.name}"
         study = create_study(storage=storage, study_name=study_name)
         assert isinstance(study._storage, storages._CachedStorage), "Pre-condition."
@@ -705,3 +679,35 @@ def test_persisted_param() -> None:
         study = load_study(storage=storage, study_name=study_name)
 
         assert all("x" in t.params for t in study.trials)
+
+
+@pytest.mark.parametrize("storage_mode", STORAGE_MODES)
+def test_lazy_trial_system_attrs(storage_mode: str) -> None:
+    with StorageSupplier(storage_mode) as storage:
+        study = optuna.create_study(storage=storage)
+        trial = study.ask()
+        storage.set_trial_system_attr(trial._trial_id, "int", 0)
+        storage.set_trial_system_attr(trial._trial_id, "str", "A")
+
+        # _LazyTrialSystemAttrs gets attrs the first time it is needed.
+        # Then, we create the instance for each method, and test the first and second use.
+
+        system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
+        assert system_attrs == {"int": 0, "str": "A"}  # type: ignore[comparison-overlap]
+        assert system_attrs == {"int": 0, "str": "A"}  # type: ignore[comparison-overlap]
+
+        system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
+        assert len(system_attrs) == 2
+        assert len(system_attrs) == 2
+
+        system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
+        assert set(system_attrs.keys()) == {"int", "str"}
+        assert set(system_attrs.keys()) == {"int", "str"}
+
+        system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
+        assert set(system_attrs.values()) == {0, "A"}
+        assert set(system_attrs.values()) == {0, "A"}
+
+        system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
+        assert set(system_attrs.items()) == {("int", 0), ("str", "A")}
+        assert set(system_attrs.items()) == {("int", 0), ("str", "A")}
