@@ -1,9 +1,8 @@
+from __future__ import annotations
+
 from io import BytesIO
 from typing import Any
 from typing import Callable
-from typing import List
-from typing import Optional
-from typing import Tuple
 
 import pytest
 
@@ -50,7 +49,6 @@ def test_target_is_none_and_study_is_multi_obj() -> None:
 def test_plot_param_importances_customized_target_name(
     plot_param_importances: Callable[..., Any]
 ) -> None:
-
     params = ["param_a", "param_b"]
     study = prepare_study_with_trials()
     figure = plot_param_importances(study, params=params, target_name="Target Name")
@@ -80,7 +78,7 @@ def test_plot_param_importances_customized_target_name(
 def test_plot_param_importances(
     plot_param_importances: Callable[..., Any],
     specific_create_study: Callable[[], Study],
-    params: Optional[List[str]],
+    params: list[str] | None,
 ) -> None:
     study = specific_create_study()
     figure = plot_param_importances(study, params=params)
@@ -88,6 +86,7 @@ def test_plot_param_importances(
         figure.write_image(BytesIO())
     else:
         plt.savefig(BytesIO())
+        plt.close()
 
 
 @pytest.mark.parametrize(
@@ -103,9 +102,8 @@ def test_plot_param_importances(
     ],
 )
 def test_get_param_importances_info_empty(
-    specific_create_study: Callable[[], Study], params: Optional[List[str]]
+    specific_create_study: Callable[[], Study], params: list[str] | None
 ) -> None:
-
     study = specific_create_study()
     info = _get_importances_info(
         study, None, params=params, target=None, target_name="Objective Value"
@@ -192,7 +190,7 @@ def test_get_info_importances_nonfinite_removed(
 def test_multi_objective_trial_with_infinite_value_ignored(
     target_idx: int, inf_value: float, evaluator: BaseImportanceEvaluator, n_trial: int
 ) -> None:
-    def _multi_objective_function(trial: Trial) -> Tuple[float, float]:
+    def _multi_objective_function(trial: Trial) -> tuple[float, float]:
         x1 = trial.suggest_float("x1", 0.1, 3)
         x2 = trial.suggest_float("x2", 0.1, 3, log=True)
         x3 = trial.suggest_float("x3", 2, 4, log=True)
